`timescale 1ns / 1ps

//////////////////////////////////////////////////////////////////////////////////
// Company:				Xilinx Inc.
// 
// Create Date:		09/02/2010 
// 
// Module Name:		My_XADC_tb 
// Project Name: 		XADC UG Example Test Bench
// Target Devices:	Virtex-7
// Tool versions:		13.0.2 Beta 2
// Description: 		Test bench for XADC UG Example design
//							
// Dependencies: 
//
// Revision: 			1.0
// Comments:			See 7 series XADC User Guide 
//
//////////////////////////////////////////////////////////////////////////////////

module My_XADC_tb;

	// Inputs
	reg VAUXP0;
	reg VAUXN0;
	reg VAUXP1;
	reg VAUXN1;
	reg VAUXP2;
	reg VAUXN2;
	reg VAUXP3;
	reg VAUXN3;
	
	reg [6:0] DADDR;
	reg DCLK;
	reg DEN, DWE;
	reg RESET;

	// Outputs
	wire [15:0] DO;
	wire DRDY;
	wire EOS;
	wire BUSY;
	wire [4:0] CHANNEL;
	wire VCCAUX_ALARM;
	wire VCCINT_ALARM;
   wire USER_TEMP_ALARM;
   wire VBRAM_ALARM;
	wire ALARM;

	// Testbench variables
	
	wire GND_BIT;
   assign GND_BIT = 0;
		
	reg [15:0] temp_output;
	reg [15:0] Vccint_output;
	reg [15:0] Vccaux_output;
	reg [15:0] Vccbram_output;
	
	
	reg [15:0] Aux0_output;
	reg [15:0] Aux1_output;
	reg [15:0] Aux2_output;
	reg [15:0] Aux3_output;
	

	// Instantiate the Unit Under Test (UUT)
	My_XADC uut (
		.VAUXP0(VAUXP0), 
		.VAUXN0(VAUXN0), 
		.VAUXP1(VAUXP1), 
		.VAUXN1(VAUXN1), 
		.VAUXP2(VAUXP2), 
		.VAUXN2(VAUXN2), 
		.VAUXP3(VAUXP3), 
		.VAUXN3(VAUXN3), 
		.DADDR_IN(DADDR), 
		.DCLK_IN(DCLK), 
		.DEN_IN(DEN),
		.DWE_IN(GND_BIT),
		.RESET_IN(RESET), 
		.DO_OUT(DO), 
		.DRDY_OUT(DRDY), 
		.EOS_OUT(EOS), 
		.BUSY_OUT(BUSY), 
		.CHANNEL_OUT(CHANNEL),
		.VCCAUX_ALARM_OUT(VCCAUX_ALARM),
		.VCCINT_ALARM_OUT(VCCINT_ALARM),
      .USER_TEMP_ALARM_OUT(USER_TEMP_ALARM),
      .VBRAM_ALARM_OUT(VBRAM_ALARM),
      .ALARM_OUT(ALARM)
	);


	// Clock Generator
	`define DCLK_MHZ      50
	always #(1000.0 / (2*`DCLK_MHZ)) DCLK  = ~DCLK;
	  
	
	initial begin
		// Initialize Inputs
		DADDR = 0;
		DCLK = 0;
		DEN = 0;
		RESET = 1;

		// Wait 4 us for global reset to finish
		repeat(200) @(posedge DCLK);
		RESET = 0;
		
		//$finish;
	end
	
	// Read back results when EOS pulses
	// indicates it's ready.
	always@(posedge EOS) begin
	
	// Read the first simultaneously sampled channel
	
	drp_read(0);		// Read the result for temp
	temp_output = DO;	// Save the result for temp
	
	drp_read(1);		// Read the result for Vccint
	Vccint_output = DO;	// Save the result for Vccint

	drp_read(2);		// Read the result for Vccaux
	Vccaux_output = DO;	// Save the result for Vccaux

	drp_read(6);		// Read the result for Vccbram
	Vccbram_output = DO;	// Save the result for Vccbram
	
	drp_read(16);		// Read the result for Aux0
	Aux0_output = DO;	// Save the result for Aux0
		
	drp_read(17);		// Read the result for Aux1
	Aux1_output = DO;	// Save the result for Aux1
	
	drp_read(18);		// Read the result for Aux2
	Aux2_output = DO;	// Save the result for Aux2

	drp_read(19);		// Read the result for Aux2
	Aux3_output = DO;	// Save the result for Aux2
	
	end
  		
	// Helper tasks 	
	// Note: this task assumes there are variables defined: DCLK, DADDR, DEN, DWE, and DRDY
	task drp_read;
	input [6:0] addr;
		begin
			@(negedge DCLK);
			DEN   = 1'b1;
			DWE   = 1'b0;
			DADDR = addr;
			@(negedge DCLK);		// DEN only pulsed for one clock cycle
			DEN   = 0;
			wait(DRDY==1);       // Wait for the drdy output to go hi before exiting.
			@(negedge DCLK);
		end
	endtask


endmodule

